#include "stdafx.h"

#include "AlphaTelex.h"

//     
UINT _ReadingPortThread(void *pParam)
{
	CAlphaTelex &at=*(CAlphaTelex *)pParam;

	for (; at.fReadInProgress; ) {
		BYTE b=0;
		if (at.ReadByte(&b)) {
			if (at.buflen-1<16384 && at.buflenmax>16384) {
				at.buflenmax-=16384;
				at.buffer=(BYTE *)realloc(at.buffer,at.buflenmax+1);
			}
			if (at.buflen+1>=at.buflenmax) {
				at.buflenmax+=16384;
				at.buffer=(BYTE *)realloc(at.buffer,at.buflenmax+1);
			}
			EnterCriticalSection(&at.critical_section);
			at.buffer[at.buflen++]=b;
			LeaveCriticalSection(&at.critical_section);
		}
	}

	return 0;
}



CAlphaTelex::CAlphaTelex()
{
	m_EventMask=EV_RXCHAR;
	m_Errors=CE_BREAK|CE_DNS|CE_FRAME|CE_IOE|CE_MODE|CE_OOP|CE_OVERRUN|CE_PTO|CE_RXOVER|CE_RXPARITY|CE_TXFULL;
	fWaitingOnRead=FALSE;
	memset(&m_ovpr,0,sizeof(m_ovpr));
	memset(&m_ovpw,0,sizeof(m_ovpw));
	fReadInProgress=FALSE;

	//   
	buflenmax=16384;
	buflen=0;
	buffer=new BYTE[buflenmax+1];
	InitializeCriticalSection(&critical_section);
}

CAlphaTelex::~CAlphaTelex()
{
	if (fReadInProgress) Disconnect();
	delete buffer;
}

BOOL CAlphaTelex::Connect(CString port)
{
	BOOL retv=TRUE;

	if (fReadInProgress) return TRUE;

	//  COM-
	hCom=CreateFile(port,GENERIC_READ|GENERIC_WRITE,
		0,						// comm devices must be opened w/exclusive-access
		NULL,					// no security attrs
		OPEN_EXISTING,			// comm devices must use OPEN_EXISTING
		FILE_FLAG_OVERLAPPED,	// overlapped I/O
		NULL					// hTemplate must be NULL for comm devices
		);
	// ,     
	if (hCom==INVALID_HANDLE_VALUE) {
		dwError=GetLastError(); Beep(3000,100);
		AfxMessageBox("  . \n"
			"    ");
		retv=FALSE; goto fin;
	}

	//   
	ClearCommError(hCom,&m_Errors,NULL);
	fSuccess=GetCommState(hCom,&dcb);
	if (!fSuccess) {
		dwError=GetLastError();
		AfxMessageBox("   . \n"
			"    ");
		retv=FALSE; goto fin;
	}

	BuildCommDCBAndTimeouts("baud=38400 parity=O data=8 stop=1 TO=OFF",&dcb,&comt);

	//    
	dcb.fBinary=TRUE;
	fSuccess=SetCommState(hCom, &dcb);
	if (!fSuccess) {
		dwError=GetLastError(); Beep(3000,100);
		AfxMessageBox("   . \n"
			"    ");
		retv=FALSE; goto fin;
	}

	SetCommTimeouts(hCom,&comt);

	fReadInProgress=TRUE;

	m_ovpr.hEvent=CreateEvent(NULL, TRUE, FALSE, NULL);
	if (m_ovpr.hEvent==NULL) {
		AfxMessageBox("CreateEvent() error");
		goto fin;
	}

	m_ovpw.hEvent=CreateEvent(NULL, TRUE, FALSE, NULL);
	if (m_ovpr.hEvent==NULL) {
		AfxMessageBox("CreateEvent() error");
		goto fin;
	}

	AfxBeginThread(_ReadingPortThread,this,THREAD_PRIORITY_LOWEST);
	Sleep(100);
fin:
	return retv;
}

BOOL CAlphaTelex::Disconnect()
{
	fReadInProgress=FALSE; Sleep(200);
	CloseHandle(m_ovpr.hEvent);
	CloseHandle(m_ovpw.hEvent);
	CloseHandle(hCom);
	return TRUE;
}


//    
BYTE CAlphaTelex::_GetModemState()
{
	cmd=0x81;

	ClearBuffer();
	WriteByte(cmd);
	return GetByte();
}

//    e  
BOOL CAlphaTelex::_PokeInternal(BYTE addr, BYTE b)
{
	cmd=0x82;

	WriteByte(cmd);
	WriteByte(addr);
	WriteByte(b);

	return TRUE;
}


//      
BOOL CAlphaTelex::_Poke(unsigned short addr, BYTE b)
{
	union {
		unsigned short addr;
		struct {
			BYTE al;
			BYTE ah;
		} byte;
	} uaddr;

	cmd=0x83;
	uaddr.addr=addr;

	WriteByte(cmd);
	WriteByte(uaddr.byte.al);
	WriteByte(uaddr.byte.ah);
	WriteByte(b);

	return TRUE;
}

//       
BOOL CAlphaTelex::_Poke(BYTE b)
{
	cmd=0x84;

	WriteByte(cmd);
	WriteByte(b);

	return TRUE;
}


//      
BYTE CAlphaTelex::_PeekInternal(BYTE addr)
{
	cmd=0x85;

	ClearBuffer();
	WriteByte(cmd);
	WriteByte(addr);
	return GetByte();
}

//      
BYTE CAlphaTelex::_Peek(unsigned short addr)
{
	cmd=0x86;

	union {
		unsigned short addr;
		struct {
			BYTE al;
			BYTE ah;
		} byte;
	} uaddr;

	uaddr.addr=addr;

	ClearBuffer();
	WriteByte(cmd);
	WriteByte(uaddr.byte.al);
	WriteByte(uaddr.byte.ah);

	return GetByte();
}

//       
BYTE CAlphaTelex::_Peek()
{
	cmd=0x87;

	ClearBuffer();
	WriteByte(cmd);
	
	return GetByte();
}


BOOL CAlphaTelex::_PokeInternalAnd(BYTE addr, BYTE b)
{
	cmd=0x89;

	WriteByte(cmd);
	WriteByte(addr);
	WriteByte(b);
	
	return TRUE;
}

BOOL CAlphaTelex::_PokeInternalOr(BYTE addr, BYTE b)
{
	cmd=0x88;

	WriteByte(cmd);
	WriteByte(addr);
	WriteByte(b);
	
	return TRUE;
}

//   
BOOL CAlphaTelex::_Reset()
{
	WriteByte('\x8a');
	WriteByte('\x8a');
	WriteByte('\x8a');
	WriteByte('\x8a');
	WriteByte('\x8a');
	Sleep(10000);		//  
	return TRUE;
}

//  FLASH
BYTE CAlphaTelex::_ClearFlash()
{
	WriteByte('\x8a');
	WriteByte('\x8a');
	WriteByte('\x8a');
	WriteByte('\x8f');
	return TRUE;
}

//  
BOOL CAlphaTelex::_SendBreak()
{
	cmd=0x8b;

	WriteByte(cmd);

	return TRUE;
}

//       
BOOL CAlphaTelex::_ManualCall()
{
	cmd=0x8c;

	WriteByte(cmd);

	return TRUE;
}

//  
BOOL CAlphaTelex::_AutoAnswer()
{
	cmd=0x8d;

	WriteByte(cmd);

	return TRUE;
}

//    
BOOL CAlphaTelex::_SendByte(BYTE b)
{
	cmd=0x8e;

	WriteByte(cmd);
	WriteByte(b);

	return TRUE;
}

//   
BOOL CAlphaTelex::_SendDigit(BYTE n)
{
	cmd=0x8f;

	WriteByte(cmd);
	WriteByte(n);

	return TRUE;
}

//    .
BOOL CAlphaTelex::_ManualScanner()
{
	cmd=0x80;

	WriteByte(cmd);

	return TRUE;
}

//  
BOOL CAlphaTelex::_DecrementRecv()
{
	cmd=0xc0;

	WriteByte(cmd);

	return TRUE;
}

//      
BOOL CAlphaTelex::SetRecvLockMode(BOOL f)
{
	AlphaFlags2C af2c; 
	
	if (f) {
		af2c.byte=0;	af2c.flags.b_Dont_Recv=TRUE;
		_PokeInternalOr(0x2c,af2c.byte);
	} else {
		af2c.byte=0xff;	af2c.flags.b_Dont_Recv=FALSE;
		_PokeInternalAnd(0x2c,af2c.byte);
	}

	return TRUE;
}

// ,      
BOOL CAlphaTelex::CanAccessBuffer()
{
	AlphaFlags2C af2c;

	af2c.byte=_PeekInternal(0x2c);

	return !af2c.flags.b_Cant_Read_Buf;
}

//    
BOOL CAlphaTelex::SetTraceMode(BOOL f)
{
	AlphaFlags2C af2c; 
	
	if (f) {
		af2c.byte=0;	af2c.flags.b_Trace_On=TRUE;
		_PokeInternalOr(0x2c,af2c.byte);
	} else {
		af2c.byte=0xff;	af2c.flags.b_Trace_On=FALSE;
		_PokeInternalAnd(0x2c,af2c.byte);
	}

	return TRUE;
}

//     
BOOL CAlphaTelex::HaveSentTlgs()
{
	AlphaFlags2D af2d;

	af2d.byte=_PeekInternal(0x2d);

	return af2d.flags.b_Have_Sent_Tlgs;
}

//     
BOOL CAlphaTelex::HaveRecvTlgs()
{
	AlphaFlags2D af2d;

	af2d.byte=_PeekInternal(0x2d);

	return af2d.flags.b_Have_Recv_Tlgs;
}

BOOL CAlphaTelex::IsModeSend()
{
	AlphaFlags2D af2d;

	af2d.byte=_PeekInternal(0x2d);

	return af2d.flags.b_Mode_Send;

}

BOOL CAlphaTelex::IsModeRecv()
{
	AlphaFlags2D af2d;

	af2d.byte=_PeekInternal(0x2d);

	return af2d.flags.b_Mode_Recv;
}

BOOL CAlphaTelex::SetIBMReadMode(BOOL f)
{
	AlphaFlags2E af2e;

	if (f) {
		af2e.byte=0;	af2e.flags.b_IBM_Read=TRUE;
		_PokeInternalOr(0x2e,af2e.byte);
	} else {
		af2e.byte=0xff;	af2e.flags.b_IBM_Read=FALSE;
		_PokeInternalAnd(0x2e,af2e.byte);
	}

	return TRUE;
}

//      
BOOL CAlphaTelex::SetModemTime(COleDateTime dtime)
{
	_PokeInternal(0x3a,dtime.GetSecond());
	_PokeInternal(0x3b,dtime.GetMinute());
	_PokeInternal(0x3c,dtime.GetHour());
	_PokeInternal(0x3d,dtime.GetDay());
	_PokeInternal(0x3e,dtime.GetMonth());
	_PokeInternal(0x3f,dtime.GetYear()-1980);
	return TRUE;
}

//    
unsigned short CAlphaTelex::GetCurrentTlgAddress()
{
	BYTE ah=_PeekInternal(0x4d);
	BYTE al=_PeekInternal(0x4e);

	return ah*256u+al;
}

//      
unsigned short CAlphaTelex::GetFreeAddress()
{
	BYTE ah=_PeekInternal(0x59);
	BYTE al=_PeekInternal(0x5a);

	return ah*256u+al;
}

//      
BOOL CAlphaTelex::SetFreeAddress(unsigned short addr)
{
	BYTE ah=addr/256;
	BYTE al=addr%256;
	_PokeInternal(0x59,ah);
	_PokeInternal(0x5a,al);
	return TRUE;
}


unsigned short CAlphaTelex::GetMemorySize()
{
	BYTE ah=_PeekInternal(0x66);
	BYTE al=_PeekInternal(0x67);

	return ah*256u+al;
}


BOOL CAlphaTelex::SetFlashOperations(BOOL f)
{
	return _PokeInternal(0x77,f);
}

BOOL CAlphaTelex::GetFlashStatus()
{
	return _PeekInternal(0x78);
}

unsigned short CAlphaTelex::GetFreeFlashAddress()
{
	BYTE ah=_PeekInternal(0x79);
	BYTE al=_PeekInternal(0x7a);

	return ah*256u+al;
}


//     15       
//                                               
BOOL CAlphaTelex::GetParamSound15Tlg()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	return bb.flags.bit0;
}

//     15       
//                                               
void CAlphaTelex::SetParamSound15Tlg(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	bb.flags.bit0=f;
	_Poke(0x0001,bb.byte);
}

//              
//                                                       
BOOL CAlphaTelex::GetParamSoundContTlg()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	return bb.flags.bit1;
}

//              
//                                                       
void CAlphaTelex::SetParamSoundContTlg(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	bb.flags.bit1=f;
	_Poke(0x0001,bb.byte);
}

//       
BOOL CAlphaTelex::GetParamSoundOnceTlg()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	return bb.flags.bit3;
}

//       
void CAlphaTelex::SetParamSoundOnceTlg(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	bb.flags.bit3=f;
	_Poke(0x0001,bb.byte);
}

//                 
//   :                                             
//   ...,   ...,   ...      
BOOL CAlphaTelex::GetParamSound_CKS_Tlg()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	return bb.flags.bit4;
}

//                 
//   :                                             
//   ...,   ...,   ...      
void CAlphaTelex::SetParamSound_CKS_Tlg(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0001);
	bb.flags.bit4=f;
	_Poke(0x0001,bb.byte);
}


//       (.)   
//         ,                
//    .                            
BYTE CAlphaTelex::GetPagamMinTlgSise4Sound()
{
	return _Peek(0x0002);
}

//       (.)   
//         ,                
//    .                            
void CAlphaTelex::SetPagamMinTlgSise4Sound(BYTE n)
{
	_Poke(0x0002,n);
}

//      .     
//    ,          
//               
// .            
//      .                           
//   127 .                              
BYTE CAlphaTelex::GetPagamSendLoopTime()
{
	return _Peek(0x0003);

}

//      .     
//    ,          
//               
// .            
//      .                           
//   127 .                              
void CAlphaTelex::SetPagamSendLoopTime(BYTE n)
{
	if (n>127) n=127;
	_Poke(0x0003,n);
}


//  / ()                      
unsigned short CAlphaTelex::GetParamLineBPS()
{
	return 5000/_Peek(0x0005);
}

//  / ()                      
void CAlphaTelex::SetParamLineBPS(unsigned short bps)
{
	if (5000%bps==0) _Poke(0x0005,5000/bps);
	else _Poke(0x0005,100);
}

//                      
//   ( ).                                    
BYTE CAlphaTelex::GetParamAutoanswerDelayText()
{
	return _Peek(0x0006);
}

//                      
//   ( ).                                    
void CAlphaTelex::SetParamAutoanswerDelayText(BYTE b)
{
	_Poke(0x0006,b);
}

//                                 
BOOL CAlphaTelex::GetParamAllowSpecProtocolSend()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0007);
	return bb.flags.bit0;
}

//                                 
void CAlphaTelex::SetParamAllowSpecProtocolSend(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0007);
	bb.flags.bit0=f;
	_Poke(0x0007,bb.byte);
}

//                                     
BOOL CAlphaTelex::GetParamAllowSpecProtocolRecv()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0007);
	return bb.flags.bit1;
}

//                                     
void CAlphaTelex::SetParamAllowSpecProtocolRecv(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0007);
	bb.flags.bit1=f;
	_Poke(0x0007,bb.byte);
}

//   -               
//    ( "")                             
BOOL CAlphaTelex::GetParamSendWhoIsIfNoAnswer()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0007);
	return bb.flags.bit2;
}

//   -               
//    ( "")                             
void CAlphaTelex::SetParamSendWhoIsIfNoAnswer(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0007);
	bb.flags.bit2=f;
	_Poke(0x0007,bb.byte);
}

//      .                      
//   :                                                 
//   -  -10 .                  
//   -  - 1                   
BYTE CAlphaTelex::GetParamTimeToFixBreak()
{
	return _Peek(0x0008);
}

//      .                      
//   :                                                 
//   -  -10 .                  
//   -  - 1                   
void CAlphaTelex::SetParamTimeToFixBreak(BYTE b)
{
	_Poke(0x0008,b);
}

//        .       
BOOL CAlphaTelex::GetParamRequireWhoIsForInCall()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0009);
	return bb.flags.bit0;
}

//        .       
void CAlphaTelex::SetParamRequireWhoIsForInCall(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0009);
	bb.flags.bit0=f;
	_Poke(0x0009,bb.byte);
}


//    ( )         
//       .                         
BOOL CAlphaTelex::GetParamWriteAutoanswerToMemoryForInCall()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0010);
	return bb.flags.bit0;
}

//    ( )         
//       .                         
void CAlphaTelex::SetParamWriteAutoanswerToMemoryForInCall(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0010);
	bb.flags.bit0=f;
	_Poke(0x0010,bb.byte);
}

//       .     
//                    
//  -     -   "" .   
//   1 .                                        
BYTE CAlphaTelex::GetParamHangProtectTimeout()
{
	return _Peek(0x0011);
}

//       .     
//                    
//  -     -   "" .   
//   1 .                                        
void CAlphaTelex::SetParamHangProtectTimeout(BYTE b)
{
	_Poke(0x0011,b);
}

//                   
//     .                          
//   1 .                                        
BYTE CAlphaTelex::GetParamPreNumberATimeout()
{
	return _Peek(0x0012);
}

//                   
//     .                          
//   1 .                                        
void CAlphaTelex::SetParamPreNumberATimeout(BYTE b)
{
	_Poke(0x0012,b);
}

//         B .        
//                   
//       .                        
//   1 .                                        
BYTE CAlphaTelex::GetParamPreNumberBTimeout()
{
	return _Peek(0x0013);
}

//         B .        
//                   
//       .                        
//   1 .                                        
void CAlphaTelex::SetParamPreNumberBTimeout(BYTE b)
{
	_Poke(0x0013,b);
}

//   ( +)     .      
//   1 .                                   
BYTE CAlphaTelex::GetParamTimePlusValueNumber()
{
	return _Peek(0x0014);
}

//   ( +)     .      
//   1 .                                   
void CAlphaTelex::SetParamTimePlusValueNumber(BYTE b)
{
	_Poke(0x0014,b);
}

//   ( -)     .   
//   1 .                                   
BYTE CAlphaTelex::GetParamTimeMinisValueNumber()
{
	return _Peek(0x0015);
}

//   ( -)     .   
//   1 .                                   
void CAlphaTelex::SetParamTimeMinusValueNumber(BYTE b)
{
	_Poke(0x0015,b);
}

//       .             
//   10 .                                   
BYTE CAlphaTelex::GetParamTimeIntervalValueNumber()
{
	return _Peek(0x0016);
}

//       .             
//   10 .                                   
void CAlphaTelex::SetParamTimeIntervalValueNumber(BYTE b)
{
	_Poke(0x0016,b);
}

//                 
//      .                   
//   1 .                                        
BYTE CAlphaTelex::GetParamDelayBeforeBreak()
{
	return _Peek(0x0017);
}

//                 
//      .                   
//   1 .                                        
void CAlphaTelex::GetParamDelayBeforeBreak(BYTE b)
{
	_Poke(0x0017,b);
}

//       .    
//   1 .                                   
BYTE CAlphaTelex::GetParamMaxTimeImpulseInviteNumber()
{
	return _Peek(0x0018);
}

//       .    
//   1 .                                   
void CAlphaTelex::SetParamMaxTimeImpulseInviteNumber(BYTE b)
{
	_Poke(0x0018,b);
}

//              
BOOL CAlphaTelex::GetParamAllowSoundOnCall()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0019);
	return bb.flags.bit0;
}

//              
void CAlphaTelex::SetParamAllowSoundOnCall(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0019);
	bb.flags.bit0=f;
	_Poke(0x0019,bb.byte);
}

//      12           
//                                                   
BOOL CAlphaTelex::GetParamAllowSoundOn12UUU_CKS()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0019);
	return bb.flags.bit1;
}

//      12           
//                                                   
void CAlphaTelex::SetParamAllowSoundOn12UUU_CKS(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0019);
	bb.flags.bit1=f;
	_Poke(0x0019,bb.byte);
}


//  
BYTE CAlphaTelex::GetParamAttemptLimit()
{
	return _Peek(0x0020);
}

//  
void CAlphaTelex::SetParamAttemptLimit(BYTE b)
{
	_Poke(0x0020,b);
}

//                          
BOOL CAlphaTelex::GetParamSendBreakOnXXXSend()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	return bb.flags.bit0;
}

//                          
void CAlphaTelex::SetParamSendBreakOnXXXSend(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	bb.flags.bit0=f;
	_Poke(0x0021,bb.byte);
}

//             
BOOL CAlphaTelex::GetParamQueueOffOnXXXSend()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	return bb.flags.bit1;
}

//             
void CAlphaTelex::SetParamQueueOffOnXXXSend(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	bb.flags.bit1=f;
	_Poke(0x0021,bb.byte);
}

//                       
BOOL CAlphaTelex::GetParamBeginRecvOnXXXSend()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	return bb.flags.bit2;
}

//                       
void CAlphaTelex::SetParamBeginRecvOnXXXSend(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	bb.flags.bit2=f;
	_Poke(0x0021,bb.byte);
}

//                 
//                                                        
BOOL CAlphaTelex::GetParamQueueOffOnBreakWhenSending()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	return bb.flags.bit3;
}

//                 
//                                                        
void CAlphaTelex::SetParamQueueOffOnBreakWhenSending(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0021);
	bb.flags.bit3=f;
	_Poke(0x0021,bb.byte);
}

//      -               
BYTE CAlphaTelex::GetParamWaitInviteForNumberA()
{
	return _Peek(0x0022);
}

//      -               
void CAlphaTelex::SetParamWaitInviteForNumberA(BYTE b)
{
	_Poke(0x0022,b);
}

//                    
//   1 .                                        
BYTE CAlphaTelex::GetParamWaitForNumberStation()
{
	return _Peek(0x0023);
}

//                    
//   1 .                                        
void CAlphaTelex::SetParamWaitForNumberStation(BYTE b)
{
	_Poke(0x0023,b);
}

//       .     
//   1 .                                        
BYTE CAlphaTelex::GetParamWaitForAutoanser()
{
	return _Peek(0x0024);
}

//       .     
//   1 .                                        
void CAlphaTelex::SetParamWaitForAutoanser(BYTE b)
{
	_Poke(0x0024,b);
}

//                 
//                                              
BYTE CAlphaTelex::GetParamDelayBeforeSendingAutoanswer()
{
	return _Peek(0x0025);
}

//                 
//                                              
void CAlphaTelex::SetParamDelayBeforeSendingAutoanswer(BYTE b)
{
	_Poke(0x0025,b);
}

//        :      
//    a)      .                
//    )                     
//   1 .                                         
BYTE CAlphaTelex::GetParamDelayOnUnsucAttempt()
{
	return _Peek(0x0026);
}

//        :      
//    a)      .                
//    )                     
//   1 .                                         
void CAlphaTelex::SetParamDelayOnUnsucAttempt(BYTE b)
{
	_Poke(0x0026,b);
}

//                                            
// 0 -                                          
// 1 -                                           
BOOL CAlphaTelex::GetParamReceiverMode()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0027);
	return bb.flags.bit0;
}

//                                            
// 0 -                                          
// 1 -                                           
void CAlphaTelex::SetParamReceiverMode(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0027);
	bb.flags.bit0=f;
	_Poke(0x0027,bb.byte);
}

//      0.5                      
BOOL CAlphaTelex::GetParamStopBitLen05()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0027);
	return bb.flags.bit1;
}

//      0.5                      
void CAlphaTelex::SetParamStopBitLen05(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0027);
	bb.flags.bit1=f;
	_Poke(0x0027,bb.byte);
}

//    ,  .                     
//   10 .                                   
BYTE CAlphaTelex::GetParamBreakLen()
{
	return _Peek(0x0028);
}

//    ,  .                     
//   10 .                                   
void CAlphaTelex::SetParamBreakLen(BYTE b)
{
	_Poke(0x0028,b);
}

//       .     
//   1                                     
BYTE CAlphaTelex::GetParamMinTimeImpulseInviteForNumber()
{
	return _Peek(0x0029);
}

//       .     
//   1                                     
void CAlphaTelex::SetParamMinTimeImpulseInviteForNumber(BYTE b)
{
	_Poke(0x0029,b);
}

//     - .                 
//                     
//       ,             
//                 
// .                                                        
BYTE CAlphaTelex::GetParamMaxModemRAMUsage()
{
	return _Peek(0x0030);
}

//     - .                 
//                     
//       ,             
//                 
// .                                                        
void CAlphaTelex::SetParamMaxModemRAMUsage(BYTE b)
{
	_Poke(0x0030,b);
}

//     -         
//  .
//   127                                                  
BYTE CAlphaTelex::GetParamLimitUneffAttemptsForOneDir()
{
	return _Peek(0x0031);
}

//     -         
//  .                                                  
//   127                                                  
void CAlphaTelex::SetParamLimitUneffAttemptsForOneDir(BYTE b)
{
	_Poke(0x0031,b);
}

//       .              
//   10 .                                   
BYTE CAlphaTelex::GetParamNumberDigitInterval()
{
	return _Peek(0x0032);
}

//       .              
//   10 .                                   
void CAlphaTelex::SetParamNumberDigitInterval(BYTE b)
{
	_Poke(0x0032,b);
}

//                 
//   1 .                                   
BYTE CAlphaTelex::GetParamMinLenSinalInUse()
{
	return _Peek(0x0033);
}

//                 
//   1 .                                   
void CAlphaTelex::SetParamMinLenSinalInUse(BYTE b)
{
	_Poke(0x0033,b);
}

//                
//   1 .                                   
BYTE CAlphaTelex::GetParamMaxLenSinalInUse()
{
	return _Peek(0x0034);
}

//                
//   1 .                                   
void CAlphaTelex::SetParamMaxLenSinalInUse(BYTE b)
{
	_Poke(0x0034,b);
}

//   :                                                 
// 0 -                                                
// 1 -                                           
// 2 -                               
BYTE CAlphaTelex::GetParamModemReceiveMode()
{
	return _Peek(0x0035);
}

//   :                                                 
// 0 -                                                
// 1 -                                           
// 2 -                               
void CAlphaTelex::SetParamModemReceiveMode(BYTE b)
{
	_Poke(0x0035,b);
}

//                     
//   .                                          
//   10 .                                   
BYTE CAlphaTelex::GetParamPauseBeforeInviteToNumber()
{
	return _Peek(0x0036);
}

//                     
//   .                                          
//   10 .                                   
void CAlphaTelex::SetParamPauseBeforeInviteToNumber(BYTE b)
{
	_Poke(0x0036,b);
}

//       .        
BOOL CAlphaTelex::GetParamAddToBufferAutoanswer()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0037);
	return bb.flags.bit0;
}

//       .        
void CAlphaTelex::SetParamAddToBufferAutoanswer(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0037);
	bb.flags.bit0=f;
	_Poke(0x0037,bb.byte);
}

//        ()          
//  .                                                    
BYTE CAlphaTelex::GetParamCKSBufferSize()
{
	return _Peek(0x0038);
}

//        ()          
//  .                                                    
void CAlphaTelex::SetParamCKSBufferSize(BYTE b)
{
	_Poke(0x0038,b);
}

//        ()          
//   .                              
BYTE CAlphaTelex::GetParamCKSPauseAsDelimiter()
{
	return _Peek(0x0039);
}

//        ()          
//   .                              
void CAlphaTelex::SetParamCKSPauseAsDelimiter(BYTE b)
{
	_Poke(0x0039,b);
}

//    '  '             
BOOL CAlphaTelex::GetParamCKSNNNNDelimiter()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit0;
}

//    '  '             
void CAlphaTelex::SetParamCKSNNNNDelimiter(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit0=f;
	_Poke(0x0040,bb.byte);
}

//    ''               
BOOL CAlphaTelex::GetParamCKSNNNDelimiter()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit1;
}

//    ''               
void CAlphaTelex::SetParamCKSNNNDelimiter(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit1=f;
	_Poke(0x0040,bb.byte);
}

//                          
BOOL CAlphaTelex::GetParamCKSAfterDelimiterTextVK()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit2;
}

//                          
void CAlphaTelex::SetParamCKSAfterDelimiterTextVK(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit2=2;
	_Poke(0x0040,bb.byte);
}

//      "   "        
BOOL CAlphaTelex::GetParamCKSDontArchivePauseError()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit3;
}

//      "   "        
void CAlphaTelex::SetParamCKSDontArchivePauseError(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit3=2;
	_Poke(0x0040,bb.byte);
}

//     .                                 
BOOL CAlphaTelex::GetParamCKSMakeFLASHProtocol()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit4;
}

//     .                                 
void CAlphaTelex::SetParamCKSMakeFLASHProtocol(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit4=2;
	_Poke(0x0040,bb.byte);
}

//     :             
// 0-   ,   1 - '  '                                   
BOOL CAlphaTelex::GetParamCKSAutoSelectCodeCombination()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit5;
}

//     :             
// 0-   ,   1 - '  '                                   
void CAlphaTelex::SetParamCKSAutoSelectCodeCombination(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit5=2;
	_Poke(0x0040,bb.byte);
}

//  -   ()        
BOOL CAlphaTelex::GetParamCKSHalfDuplexMode()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit6;
}

//  -   ()        
void CAlphaTelex::SetParamCKSHalfDuplexMode(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit6=2;
	_Poke(0x0040,bb.byte);
}

BOOL CAlphaTelex::GetParamCKSBlockXXXTransmitOnHalfDuplex()
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	return bb.flags.bit7;
}

void CAlphaTelex::SetParamCKSBlockXXXTransmitOnHalfDuplex(BOOL f)
{
	AlphaFlagsByte bb;
	bb.byte=_Peek(0x0040);
	bb.flags.bit7=2;
	_Poke(0x0040,bb.byte);
}

//                    
//     R30,                
//   .    -          
//                  
//    .                                
BYTE CAlphaTelex::GetParamCKSSignalToUploadNewOutgoing()
{
	return _Peek(0x0041);
}

//                    
//     R30,                
//   .    -          
//                  
//    .                                
void CAlphaTelex::SetParamCKSSignalToUploadNewOutgoing(BYTE b)
{
	_Poke(0x0041,b);
}

//                      
//   ( ).                                 
BYTE CAlphaTelex::GetParamCKSAbendTime()
{
	return _Peek(0x0041);
}

//                      
//   ( ).                                 
void CAlphaTelex::SetParamCKSAbendTime(BYTE b)
{
	_Poke(0x0041,b);
}

//         .      
//   10 .                                   
BYTE CAlphaTelex::GetParamCKSCallSignalLen()
{
	return _Peek(0x0042);
}

//         .      
//   10 .                                   
void CAlphaTelex::SetParamCKSCallSignalLen(BYTE b)
{
	_Poke(0x0042,b);
}



//     
BYTE CAlphaTelex::GetByte()
{
	BYTE b;
	GetBufferByte(&b,TRUE);
	return b;
}

//     
BOOL CAlphaTelex::ReadByte(BYTE *pb)
{
	BYTE &b=*pb;
	DWORD dw=0;

	b=0;

	if (!ReadFile(hCom,&b,1,&dw,&m_ovpr)) {
		if (GetLastError()==ERROR_IO_PENDING) {
			if (!GetOverlappedResult(hCom,&m_ovpr,&dw,TRUE)) {
				CString cs; cs.Format("%d",GetLastError());
				AfxMessageBox("GetOverlappedResult() error "+cs);
			} else dw=1;
		}
	}

	return dw;
}

//     
BOOL CAlphaTelex::WriteByte(BYTE b)
{
	DWORD dw=0;

	if (!WriteFile(hCom,&b,1,&dw,&m_ovpw)) {
		if (GetLastError()==ERROR_IO_PENDING) {
			if (!GetOverlappedResult(hCom,&m_ovpw,&dw,TRUE)) {
				CString cs; cs.Format("%d",GetLastError());
				AfxMessageBox("GetOverlappedResult() error "+cs);
			}
		} else {
			CString cs; cs.Format("%d",GetLastError());
			AfxMessageBox("WriteFile() error "+cs);
		}
	}

	return TRUE;
}


BOOL CAlphaTelex::GetBufferByte(BYTE *b, BOOL wait)
{
	BOOL retv=TRUE;


	if (wait) {
		for (; buflen<1; ) Sleep(1);
		EnterCriticalSection(&critical_section);
		*b=buffer[0];
		for (int i=1; i<(int)buflen; i++) 
			buffer[i-1]=buffer[i];
		buflen--;
		LeaveCriticalSection(&critical_section);
	} else {	//  
		if (buflen<1) retv=FALSE;
		else {
			EnterCriticalSection(&critical_section);
			*b=buffer[0];
			for (int i=1; i<(int)buflen; i++)
				buffer[i-1]=buffer[i];
			buflen--;
			LeaveCriticalSection(&critical_section);
		}
	}

	return retv;
}


void CAlphaTelex::ClearBuffer()
{
	EnterCriticalSection(&critical_section);
	buflen=0;
	LeaveCriticalSection(&critical_section);
}
